<?php

namespace InSegment\ApiCore\Models;

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;

class UUIDGeneration extends SupplementaryTable
{
    const TABLE_CAPACITY = 419424;
    
    /**
     * Run the migrations.
     *
     * @param array $enabledTables
     * @return void
     */
    public static function establish(array $enabledTables, int $maxRecords)
    {
        Schema::create((new static)->getTable(), function (Blueprint $table) use ($enabledTables, $maxRecords) {
            if ($maxRecords <= self::TABLE_CAPACITY) {
                $table->engine = 'MEMORY';
            }
            
            $table->unsignedInteger('id')->nullable();
            $table->unsignedBigInteger('uuid');
            $table->enum('table', $enabledTables);
            $table->primary('uuid');
            $table->index('table', 'uuid_generation_table_index');
        });
    }
    
    /**
     * Get statement generating amount if ids for table and key
     * 
     * @param string $table
     * @param int $rows
     * @return string
     */
    public static function getGeneratorStatement(string $table, int $rows)
    {
        $myTable = (new static)->getTable();
        $nRows = implode(', ', array_fill(0, $rows, '()'));
        
        return "CREATE TRIGGER `trigger_{$myTable}` BEFORE INSERT ON `{$myTable}`"
            . " FOR EACH ROW SET NEW.`uuid` = UUID_SHORT(), NEW.`table` = '{$table}';"
            . "INSERT INTO `{$myTable}` () VALUES {$nRows};"
            . "DROP TRIGGER `trigger_{$myTable}`;";
    }
}
