<?php

namespace InSegment\ApiCore\Providers;

use Illuminate\Support\ServiceProvider;

use InSegment\ApiCore\Console\Commands\TransactionTableCommand;
use InSegment\ApiCore\Console\Commands\EnumerateApiJsonUniques;
use InSegment\ApiCore\Console\Commands\EnumerateApiJsonRelations;
use InSegment\ApiCore\Console\Commands\GarbageCollectTransactions;

class ApiCoreServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap the application services.
     *
     * @return void
     */
    public function boot()
    {
        $configPath = __DIR__ . '/../../config/api_core.php';
        if (function_exists('config_path')) {
            $publishPath = config_path('api_core.php');
        } else {
            $publishPath = base_path('config/api_core.php');
        }
        $this->publishes([$configPath => $publishPath], 'config');
    }
    
    /**
     * Register the application services.
     *
     * @return void
     */
    public function register()
    {
        $configPath = __DIR__ . '/../../config/api_core.php';
        $this->mergeConfigFrom($configPath, 'api_core');

        $this->app->singleton('command.api.transactionTable', TransactionTableCommand::class);
        $this->app->singleton('command.api.enumerateJsonUniques', EnumerateApiJsonUniques::class);
        $this->app->singleton('command.api.enumerateJsonRelations', EnumerateApiJsonRelations::class);
        $this->app->singleton('command.api.garbageCollectTransactions', GarbageCollectTransactions::class);
        
        $this->commands([
            'command.api.transactionTable',
            'command.api.enumerateJsonUniques',
            'command.api.enumerateJsonRelations',
            'command.api.garbageCollectTransactions'
        ]);
    }
    
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [
            'command.api.enumerateJsonUniques',
            'command.api.enumerateJsonRelations',
            'command.api.garbageCollectTransactions'
        ];
    }
}
